

class WebDataSource(object):
    """ A WebDataSource performs search queries against Web Sources such as Youtube, Twitter, ... """

    def search(self, search_terms):
        """ returns the count for the given tags
            @param search_terms: a list of search terms
            @returns a list of matching documents """
        return NotImplemented
