""" helper utilities required to create databases
    for the modules in this package
    
    (*) create-distance-db.py
        - transforms wikipedia dumps into the database
          format required for the distance module.
"""