""" @package eWRT.ontology.eval
    Evaluates ontologies based on their _internal_ characteristics such as 
    term coherence, internal integrity, ...

    For evaluations against a reference ontology
    @see eWRT.ontology.compare
"""
